@@ -37,19 +37,30 @@ from oslo_config import cfg
 import stevedore.named  # noqa
 
 LOG = logging.getLogger(__name__)
+UPPER_CASE_GROUP_NAMES = ['DEFAULT']
 
 _generator_opts = [
-    cfg.StrOpt('output-file',
-               help='Path of the file to write to. Defaults to stdout.'),
-    cfg.IntOpt('wrap-width',
-               default=70,
-               help='The maximum length of help lines.'),
-    cfg.MultiStrOpt('namespace',
-                    required=True,
-                    help='Option namespace under "oslo.config.opts" in which '
-                         'to query for options.'),
-    cfg.BoolOpt('minimal', default=False,
-                help='Generate a minimal required configuration.'),
+    cfg.StrOpt(
+        'output-file',
+        help='Path of the file to write to. Defaults to stdout.'),
+    cfg.IntOpt(
+        'wrap-width',
+        default=70,
+        help='The maximum length of help lines.'),
+    cfg.MultiStrOpt(
+        'namespace',
+        required=True,
+        help='Option namespace under "oslo.config.opts" in which to query '
+        'for options.'),
+    cfg.BoolOpt(
+        'minimal',
+        default=False,
+        help='Generate a minimal required configuration.'),
+    cfg.BoolOpt(
+        'summarize',
+        default=False,
+        help='Only output summaries of help text to config files. Retain '
+        'longer help text for Sphinx documents.'),
 ]
 
 
@@ -80,7 +91,8 @@ def _format_defaults(opt):
         elif opt.default is None:
             default_str = '<None>'
         elif (isinstance(opt, (cfg.StrOpt, cfg.IPOpt,
-                               cfg.HostnameOpt, cfg.URIOpt))):
+                               cfg.HostnameOpt, cfg.HostAddressOpt,
+                               cfg.URIOpt))):
             default_str = opt.default
         elif isinstance(opt, cfg.BoolOpt):
             default_str = str(opt.default).lower()
@@ -173,6 +185,7 @@ class _OptFormatter(object):
         """Format the description of a group header to the output file
 
         :param group_or_groupname: a cfg.OptGroup instance or a name of group
+        :returns: a formatted group description string
         """
         if isinstance(group_or_groupname, cfg.OptGroup):
             group = group_or_groupname
@@ -184,10 +197,14 @@ class _OptFormatter(object):
             lines = ['[%s]\n' % groupname]
         self.writelines(lines)
 
-    def format(self, opt, group_name, minimal=False):
+    def format(self, opt, group_name, minimal=False, summarize=False):
         """Format a description of an option to the output file.
 
         :param opt: a cfg.Opt instance
+        :param group_name: name of the group to which the opt is assigned
+        :param minimal: enable option by default, marking it as required
+        :param summarize: output a summarized description of the opt
+        :returns: a formatted opt description string
         """
         if not opt.help:
             LOG.warning(_LW('"%s" is missing a help string'), opt.dest)
@@ -199,8 +216,19 @@ class _OptFormatter(object):
             opt_prefix = 'DEPRECATED: '
 
         if opt.help:
+            # an empty line signifies a new paragraph. We only want the
+            # summary line
+            if summarize:
+                _split = opt.help.split('\n\n')
+                opt_help = _split[0].rstrip(':').rstrip('.')
+                if len(_split) > 1:
+                    opt_help += '. For more information, refer to the '
+                    opt_help += 'documentation.'
+            else:
+                opt_help = opt.help
+
             help_text = u'%s%s (%s)' % (opt_prefix,
-                                        opt.help,
+                                        opt_help,
                                         opt_type)
         else:
             help_text = u'(%s)' % opt_type
@@ -259,6 +287,13 @@ class _OptFormatter(object):
                 lines.extend(
                     self._format_help('Reason: ' + opt.deprecated_reason))
 
+        if opt.advanced:
+            lines.append(
+                '# Advanced Option: intended for advanced users and not used\n'
+                '# by the majority of users, and might have a significant\n'
+                '# effect on stability and/or performance.\n'
+            )
+
         if hasattr(opt.type, 'format_defaults'):
             defaults = opt.type.format_defaults(opt.default,
                                                 opt.sample_default)
@@ -313,6 +348,22 @@ def _cleanup_opts(read_opts):
         if namespace not in clean:
             clean[namespace] = collections.OrderedDict()
         for group, opts in listing:
+            # NOTE: Normalize group names to lowe-case except those defined in
+            # UPPER_CASE_GROUP_NAMES
+            if group:
+                group_name = getattr(group, 'name', str(group))
+                if group_name.upper() in UPPER_CASE_GROUP_NAMES:
+                    normalized_gn = group_name.upper()
+                else:
+                    normalized_gn = group_name.lower()
+                if normalized_gn != group_name:
+                    LOG.warning('normalizing group name %r to %r', group_name,
+                                normalized_gn)
+                    if hasattr(group, 'name'):
+                        group.name = normalized_gn
+                    else:
+                        group = normalized_gn
+
             if group not in clean[namespace]:
                 clean[namespace][group] = collections.OrderedDict()
             for opt in opts:
@@ -321,8 +372,8 @@ def _cleanup_opts(read_opts):
     # recreate the list of (namespace, [(group, [opt_1, opt_2])]) tuples
     # from the cleaned structure.
     cleaned_opts = [
-        (namespace, [(group, list(clean[namespace][group].values()))
-                     for group in clean[namespace]])
+        (namespace, [(g, list(clean[namespace][g].values()))
+                     for g in clean[namespace]])
         for namespace in clean
     ]
 
@@ -347,6 +398,7 @@ def _get_opt_default_updaters(namespaces):
     mgr = stevedore.named.NamedExtensionManager(
         'oslo.config.opts.defaults',
         names=namespaces,
+        warn_on_missing_entrypoint=False,
         on_load_failure_callback=on_load_failure_callback,
         invoke_on_load=False)
     return [ep.plugin for ep in mgr]
@@ -384,18 +436,18 @@ def on_load_failure_callback(*args, **kwargs):
     raise
 
 
-def _output_opts(f, group, group_data, minimal=False):
+def _output_opts(f, group, group_data, minimal=False, summarize=False):
     f.format_group(group_data['object'] or group)
     for (namespace, opts) in sorted(group_data['namespaces'],
                                     key=operator.itemgetter(0)):
         f.write('\n#\n# From %s\n#\n' % namespace)
-        for opt in opts:
+        for opt in sorted(opts, key=operator.attrgetter('advanced')):
             try:
                 if minimal and not opt.required:
                     pass
                 else:
                     f.write('\n')
-                    f.format(opt, group, minimal)
+                    f.format(opt, group, minimal, summarize)
             except Exception as err:
                 f.write('# Warning: Failed to format sample for %s\n' %
                         (opt.dest,))
@@ -454,12 +506,14 @@ def generate(conf):
     groups = _get_groups(_list_opts(conf.namespace))
 
     # Output the "DEFAULT" section as the very first section
-    _output_opts(formatter, 'DEFAULT', groups.pop('DEFAULT'), conf.minimal)
+    _output_opts(formatter, 'DEFAULT', groups.pop('DEFAULT'), conf.minimal,
+                 conf.summarize)
 
     # output all other config sections with groups in alphabetical order
     for group, group_data in sorted(groups.items()):
         formatter.write('\n\n')
-        _output_opts(formatter, group, group_data, conf.minimal)
+        _output_opts(formatter, group, group_data, conf.minimal,
+                     conf.summarize)
 
 
 def main(args=None):
